@@ -1692,10 +1692,16 @@ expoInitScreen (CompPlugin *p,
     EXPO_DISPLAY (s->display);
 
     es = malloc (sizeof (ExpoScreen));
-
     if (!es)
 	return FALSE;
 
+    es->vpNormals = malloc (360 * 3 * sizeof (GLfloat));
+    if (!es->vpNormals)
+    {
+	free (es);
+	return FALSE;
+    }
+
     es->tmpRegion = XCreateRegion ();
     if (!es->tmpRegion)
     {
@@ -1724,10 +1730,6 @@ expoInitScreen (CompPlugin *p,
     es->vpActivity     = NULL;
     es->vpActivitySize = 0;
 
-    es->vpNormals = malloc (360 * 3 * sizeof (GLfloat));
-    if (!es->vpNormals)
-	return FALSE;
-
     es->winNormals  = NULL;
     es->winNormSize = 0;
 
@@ -1761,6 +1763,15 @@ expoFiniScreen (CompPlugin *p,
 
     XDestroyRegion (es->tmpRegion);
 
+    if (es->vpNormals)
+	free (es->vpNormals);
+
+    if (es->winNormals)
+	free (es->winNormals);
+
+    if (es->vpActivity)
+	free (es->vpActivity);
+
     UNWRAP (es, s, paintOutput);
     UNWRAP (es, s, paintScreen);
     UNWRAP (es, s, donePaintScreen);
