@@ -320,8 +320,16 @@ shiftRenderWindowTitle (CompScreen *s)
     tA.color[3] = shiftGetTitleFontColorAlpha (s);
     tA.style = (shiftGetTitleFontBold (s)) ?
 	       TEXT_STYLE_BOLD : TEXT_STYLE_NORMAL;
+    tA.style |= TEXT_STYLE_BACKGROUND;
     tA.family = "Sans";
     tA.ellipsize = TRUE;
+    tA.backgroundHMargin = 15;
+    tA.backgroundVMargin = 15;
+    tA.backgroundColor[0] = shiftGetTitleBackColorRed (s);
+    tA.backgroundColor[1] = shiftGetTitleBackColorGreen (s);
+    tA.backgroundColor[2] = shiftGetTitleBackColorBlue (s);
+    tA.backgroundColor[3] = shiftGetTitleBackColorAlpha (s);
+
 
     if (ss->type == ShiftTypeAll)
 	tA.renderMode = TextRenderWindowTitleWithViewport;
@@ -412,58 +420,6 @@ shiftDrawWindowTitle (CompScreen *s)
 	glEnable (GL_BLEND);
     glBlendFunc (GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
 
-    glColor4us (shiftGetTitleBackColorRed (s),
-		shiftGetTitleBackColorGreen (s),
-		shiftGetTitleBackColorBlue (s),
-		shiftGetTitleBackColorAlpha (s));
-
-    glPushMatrix ();
-
-    glTranslatef (x, y - height, 0.0f);
-    glRectf (0.0f, height, width, 0.0f);
-    glRectf (0.0f, 0.0f, width, -border);
-    glRectf (0.0f, height + border, width, height);
-    glRectf (-border, height, 0.0f, 0.0f);
-    glRectf (width, height, width + border, 0.0f);
-    glTranslatef (-border, -border, 0.0f);
-
-#define CORNER(a,b) \
-    for (k = a; k < b; k++) \
-    {\
-	float rad = k * (PI / 180.0f);\
-	glVertex2f (0.0f, 0.0f);\
-	glVertex2f (cos(rad) * border, sin(rad) * border);\
-	glVertex2f (cos((k-1) * (PI / 180.0f)) * border, \
-		    sin((k-1) * (PI / 180.0f)) * border);\
-    }
-
-    /* Rounded corners */
-    int k;
-
-    glTranslatef (border, border, 0.0f);
-    glBegin (GL_TRIANGLES);
-    CORNER (180, 270) glEnd ();
-    glTranslatef (-border, -border, 0.0f);
-
-    glTranslatef (width + border, border, 0.0f);
-    glBegin (GL_TRIANGLES);
-    CORNER (270, 360) glEnd ();
-    glTranslatef (-(width + border), -border, 0.0f);
-
-    glTranslatef (border, height + border, 0.0f);
-    glBegin (GL_TRIANGLES);
-    CORNER (90, 180) glEnd ();
-    glTranslatef (-border, -(height + border), 0.0f);
-
-    glTranslatef (width + border, height + border, 0.0f);
-    glBegin (GL_TRIANGLES);
-    CORNER (0, 90) glEnd ();
-    glTranslatef (-(width + border), -(height + border), 0.0f);
-
-    glPopMatrix ();
-
-#undef CORNER
-
     glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
     glColor4f (1.0, 1.0, 1.0, 1.0);
 
@@ -703,11 +659,7 @@ shiftPaintWindow (CompWindow		 *w,
 		    CompTransform  wTransform = *transform;
 
 		    if (!w->texture->pixmap)
-		    {
-			/* the fade plugin does weird things to
-			   w->paint.opacity, so better use the atom value */
-			sAttrib.opacity = w->opacity;
-		    }
+			sAttrib.opacity = w->paint.opacity;
 
 		    initFragmentAttrib (&fragment, attrib);
 
@@ -1467,6 +1419,9 @@ shiftPaintOutput (CompScreen		  *s,
 	{
 	    CompTransform  rTransform = sTransform;
 	    unsigned short color[4];
+	    int            cull, cullInv;
+	    glGetIntegerv (GL_CULL_FACE_MODE, &cull);
+	    cullInv = (cull == GL_BACK)? GL_FRONT : GL_BACK;
 
 	    matrixTranslate (&rTransform, 0.0, oy1 + oy2 + maxThumbHeight,
 			     0.0);
@@ -1475,7 +1430,7 @@ shiftPaintOutput (CompScreen		  *s,
 	    glPushMatrix ();
 	    glLoadMatrixf (rTransform.m);
 
-	    glDisable (GL_CULL_FACE);
+	    glCullFace (cullInv);
 
 	    if (shiftGetMipmaps (s))
 		s->display->textureFilter = GL_LINEAR_MIPMAP_LINEAR;
@@ -1501,7 +1456,7 @@ shiftPaintOutput (CompScreen		  *s,
 	    }
 
 	    glDisable (GL_CLIP_PLANE0);
-	    glEnable( GL_CULL_FACE);
+	    glCullFace (cull);
 
 	    glLoadIdentity();
 	    glTranslatef (0.0, 0.0, -DEFAULT_Z_CAMERA);
@@ -2461,7 +2416,7 @@ shiftInitDisplay (CompPlugin  *p,
 
     sd->textAvailable = checkPluginABI ("text", TEXT_ABIVERSION);
     if (!sd->textAvailable)
-	compLogMessage (d, "shift", CompLogLevelWarn,
+	compLogMessage ("shift", CompLogLevelWarn,
 			"No compatible text plugin loaded.");
 
     sd->leftKey  = XKeysymToKeycode (d->display, XStringToKeysym ("Left"));
