@@ -113,6 +113,8 @@ typedef struct _WallScreen
 
     Window moveWindow;
 
+    Bool focusDefault;
+
     Bool              miniScreen;
     WindowPaintAttrib mSAttribs;
     float             mSzCamera;
@@ -407,7 +409,7 @@ wallSetupCairoContext (CompScreen       *s,
     if (!bindPixmapToTexture(s, &context->texture, context->pixmap,
 			     width, height, 32))
     {
-	compLogMessage (s->display, "wall", CompLogLevelError,
+	compLogMessage ("wall", CompLogLevelError,
 			"Couldn't create cairo context for switcher");
     }
 
@@ -508,67 +510,60 @@ wallMoveViewport (CompScreen *s,
     if (!x && !y)
 	return FALSE;
 
-    if (wallCheckDestination (s, x, y))
-    {
-	if (ws->moveWindow != moveWindow)
-	{
-	    CompWindow *w;
+    if (otherScreenGrabExist (s, "move", "switcher", "group-drag", "wall", 0))
+	return FALSE;
 
-	    wallReleaseMoveWindow (s);
-	    w = findWindowAtScreen (s, moveWindow);
-	    if (w)
-    	    {
-		if (!(w->type & (CompWindowTypeDesktopMask |
-				 CompWindowTypeDockMask)))
+    if (!wallCheckDestination (s, x, y))
+	return FALSE;
+
+    if (ws->moveWindow != moveWindow)
+    {
+	CompWindow *w;
+
+	wallReleaseMoveWindow (s);
+	w = findWindowAtScreen (s, moveWindow);
+	if (w)
+	{
+	    if (!(w->type & (CompWindowTypeDesktopMask |
+			     CompWindowTypeDockMask)))
+	    {
+		if (!(w->state & CompWindowStateStickyMask))
 		{
-		    if (!(w->state & CompWindowStateStickyMask))
-		    {
-			ws->moveWindow = w->id;
-			ws->moveWindowX = w->attrib.x;
-			ws->moveWindowY = w->attrib.y;
-			raiseWindow (w);
-		    }
+		    ws->moveWindow = w->id;
+		    ws->moveWindowX = w->attrib.x;
+		    ws->moveWindowY = w->attrib.y;
+		    raiseWindow (w);
 		}
 	    }
 	}
-
-	if (!ws->moving)
-	{
-	    ws->curPosX = s->x;
-	    ws->curPosY = s->y;
-	}
-	ws->gotoX = s->x - x;
-	ws->gotoY = s->y - y;
-
-	if (!ws->grabIndex)
-	    ws->grabIndex = pushScreenGrab (s, s->invisibleCursor, "wall");
-
-	moveScreenViewport (s, x, y, TRUE);
-
-	ws->moving = TRUE;
-	ws->boxOutputDevice = outputDeviceForPoint (s, pointerX, pointerY);
     }
 
-    if (ws->moving)
+    if (!ws->moving)
     {
-	if (wallGetShowSwitcher (s->display))
-	    ws->boxTimeout = wallGetPreviewTimeout (s->display) * 1000;
-	else
-	    ws->boxTimeout = 0;
-
-	if (otherScreenGrabExist (s, "move", "scale", "switcher",
-				  "group-drag", "wall", 0))
-	{
-	    ws->boxTimeout = 0;
-	    ws->moving = FALSE;
-	}
-
-	ws->timer = wallGetSlideDuration (s->display) * 1000;
+	ws->curPosX = s->x;
+	ws->curPosY = s->y;
     }
+    ws->gotoX = s->x - x;
+    ws->gotoY = s->y - y;
+
+    if (!ws->grabIndex)
+	ws->grabIndex = pushScreenGrab (s, s->invisibleCursor, "wall");
+
+    moveScreenViewport (s, x, y, TRUE);
+
+    ws->moving = TRUE;
+    ws->boxOutputDevice = outputDeviceForPoint (s, pointerX, pointerY);
+
+    if (wallGetShowSwitcher (s->display))
+	ws->boxTimeout = wallGetPreviewTimeout (s->display) * 1000;
+    else
+	ws->boxTimeout = 0;
+
+    ws->timer = wallGetSlideDuration (s->display) * 1000;
 
     damageScreen (s);
 
-    return ws->moving;
+    return TRUE;
 }
 
 static void
@@ -589,7 +584,7 @@ wallHandleEvent (CompDisplay *d,
 	    if (!s)
 		break;
 
-	    if (otherScreenGrabExist (s, "switcher", "scale", 0))
+	    if (otherScreenGrabExist (s, "switcher", 0))
 		break;
 
     	    dx = event->xclient.data.l[0] / s->width - s->x;
@@ -624,7 +619,10 @@ wallActivateWindow (CompWindow *w)
 	dy -= s->y;
 	
 	if (dx || dy)
+	{
 	    wallMoveViewport (s, -dx, -dy, None);
+	    ws->focusDefault = FALSE;
+	}
     }
 
     UNWRAP (ws, s, activateWindow);
@@ -632,44 +630,6 @@ wallActivateWindow (CompWindow *w)
     WRAP (ws, s, activateWindow, wallActivateWindow);
 }
 
-static Bool
-wallNext (CompDisplay     *d,
-	  CompAction      *action,
-	  CompActionState state,
-	  CompOption      *option,
-	  int             nOption)
-{
-    GET_SCREEN;
-
-    if ((s->x == s->hsize - 1) && (s->y == s->vsize - 1))
-	wallMoveViewport (s, s->hsize - 1, s->vsize - 1, None);
-    else if (s->x == s->hsize - 1)
-	wallMoveViewport (s, s->hsize - 1, -1, None);
-    else
-	wallMoveViewport (s, -1, 0, None);
-
-    return TRUE;
-}
-
-static Bool
-wallPrev (CompDisplay     *d,
-	  CompAction      *action,
-	  CompActionState state,
-	  CompOption      *option,
-	  int             nOption)
-{
-    GET_SCREEN;
-
-    if ((s->x == 0) && (s->y == 0))
-	wallMoveViewport (s, -(s->hsize - 1), -(s->vsize - 1), None);
-    else if (s->x == 0)
-	wallMoveViewport (s, -(s->hsize - 1), 1, None);
-    else
-	wallMoveViewport (s, 1, 0, None);
-
-    return TRUE;
-}
-
 static void
 wallCheckAmount (CompScreen *s,
 		 int        dx,
@@ -707,7 +667,8 @@ wallInitiate (CompScreen      *s,
     WALL_SCREEN (s);
 
     wallCheckAmount (s, dx, dy, &amountX, &amountY);
-    wallMoveViewport (s, amountX, amountY, win);
+    if (!wallMoveViewport (s, amountX, amountY, win))
+	return TRUE;
 
     if (state & CompActionStateInitKey)
 	action->state |= CompActionStateTermKey;
@@ -716,6 +677,7 @@ wallInitiate (CompScreen      *s,
 	action->state |= CompActionStateTermButton;
 
     ws->showPreview = TRUE;
+    ws->focusDefault = TRUE;
 
     return TRUE;
 }
@@ -893,6 +855,64 @@ wallDown (CompDisplay     *d,
     return wallInitiate (s, 0, 1, None, action, state);
 }
 
+static Bool
+wallNext (CompDisplay     *d,
+	  CompAction      *action,
+	  CompActionState state,
+	  CompOption      *option,
+	  int             nOption)
+{
+    int amountX, amountY;
+    GET_SCREEN;
+
+    if ((s->x == s->hsize - 1) && (s->y == s->vsize - 1))
+    {
+	amountX = -(s->hsize - 1);
+	amountY = -(s->vsize - 1);
+    }
+    else if (s->x == s->hsize - 1)
+    {
+	amountX = -(s->hsize - 1);
+	amountY = 1;
+    }
+    else
+    {
+	amountX = 1;
+	amountY = 0;
+    }
+
+    return wallInitiate (s, amountX, amountY, None, action, state);
+}
+
+static Bool
+wallPrev (CompDisplay     *d,
+	  CompAction      *action,
+	  CompActionState state,
+	  CompOption      *option,
+	  int             nOption)
+{
+    int amountX, amountY;
+    GET_SCREEN;
+
+    if ((s->x == 0) && (s->y == 0))
+    {
+	amountX = s->hsize - 1;
+	amountY = s->vsize - 1;
+    }
+    else if (s->x == 0)
+    {
+	amountX = s->hsize - 1;
+	amountY = -1;
+    }
+    else
+    {
+	amountX = -1;
+	amountY = 0;
+    }
+
+    return wallInitiate (s, amountX, amountY, None, action, state);
+}
+
 static Bool
 wallFlipLeft (CompDisplay     *d,
 	      CompAction      *action,
@@ -1306,30 +1326,33 @@ wallPaintOutput (CompScreen              *s,
 	if (wallGetMiniscreen (s->display))
 	{
 	    int i, j;
+	    float mw, mh;
+
+	    mw = ws->viewportWidth;
+	    mh = ws->viewportHeight;
+
+	    ws->miniScreen = TRUE;
+	    ws->mSAttribs.xScale = mw / s->width;
+	    ws->mSAttribs.yScale = mh / s->height;
+	    ws->mSAttribs.opacity = OPAQUE * (1.0 + ws->mSzCamera);
+	    ws->mSAttribs.saturation = COLOR;
 
 	    for (j = 0; j < s->vsize; j++)
     	    {
 		for (i = 0; i < s->hsize; i++)
 		{
-		    float mx, my, mw, mh;
+		    float mx, my;
 		    unsigned int msMask;
 
-		    ws->miniScreen = TRUE;
 		    mx = ws->firstViewportX +
 			 (i * (ws->viewportWidth + ws->viewportBorder));
-    		    my = ws->firstViewportY +
+    		    my = ws->firstViewportY + 
 			 (j * (ws->viewportHeight + ws->viewportBorder));
-    		    mw = ws->viewportWidth;
-		    mh = ws->viewportHeight;
 
 		    ws->mSAttribs.xTranslate = mx / output->width;
 		    ws->mSAttribs.yTranslate = -my / output->height;
 
-		    ws->mSAttribs.xScale = mw / s->width;
-    		    ws->mSAttribs.yScale = mh / s->height;
-		    ws->mSAttribs.opacity = OPAQUE * (1.0 + ws->mSzCamera);
 		    ws->mSAttribs.brightness = 0.4f * BRIGHT;
-		    ws->mSAttribs.saturation = COLOR;
 
 		    if (i == s->x && j == s->y && ws->moving)
 			ws->mSAttribs.brightness = BRIGHT;
@@ -1347,10 +1370,10 @@ wallPaintOutput (CompScreen              *s,
 		    (*s->paintTransformedOutput) (s, sAttrib, transform,
 						  region, output, msMask);
 
-		    ws->miniScreen = FALSE;
 
 		}
 	    }
+	    ws->miniScreen = FALSE;
 	    setWindowPaintOffset (s, 0, 0);
 	}
     }
@@ -1401,13 +1424,17 @@ wallPreparePaintScreen (CompScreen *s,
 
 	if (ws->moveWindow)
 	    wallReleaseMoveWindow (s);
-	else
-	    focusDefaultWindow (s);
-
-	if (ws->grabIndex)
+	else if (ws->focusDefault)
 	{
-	    removeScreenGrab (s, ws->grabIndex, NULL);
-	    ws->grabIndex = 0;
+	    int i;
+	    for (i = 0; i < s->maxGrab; i++)
+		if (s->grabs[i].active)
+		    if (strcmp(s->grabs[i].name, "switcher") == 0)
+			break;
+
+	    /* only focus default window if switcher is not active */
+	    if (i == s->maxGrab)
+		focusDefaultWindow (s);
 	}
     }
 
@@ -1434,9 +1461,12 @@ wallPaintTransformedOutput (CompScreen              *s,
 	mask &= ~PAINT_SCREEN_CLEAR_MASK;
 
 	/* move each screen to the correct output position */
+
 	matrixTranslate (&sTransform,
-			 -output->region.extents.x1 / output->width,
-			 output->region.extents.y1 / output->height, 0.0f);
+			 -(float) output->region.extents.x1 /
+			  (float) output->width,
+			 (float) output->region.extents.y1 /
+			 (float) output->height, 0.0f);
 	matrixTranslate (&sTransform, 0.0f, 0.0f, -DEFAULT_Z_CAMERA);
 
 	matrixTranslate (&sTransform,
@@ -1453,8 +1483,10 @@ wallPaintTransformedOutput (CompScreen              *s,
 	   Now all screens display the same */
 	matrixTranslate (&sTransform, 0.5f, 0.5f, DEFAULT_Z_CAMERA);
 	matrixTranslate (&sTransform,
-			 output->region.extents.x1 / output->width,
-			 -output->region.extents.y2 / output->height, 0.0f);
+			 (float) output->region.extents.x1 /
+			 (float) output->width,
+			 -(float) output->region.extents.y2 /
+			 (float) output->height, 0.0f);
 
 	UNWRAP (ws, s, paintTransformedOutput);
 	(*s->paintTransformedOutput) (s, sAttrib, &sTransform,
@@ -1602,14 +1634,17 @@ wallDonePaintScreen (CompScreen *s)
     WALL_SCREEN (s);
 
     if (ws->moving || ws->boxTimeout)
-	damageScreen (s);
-
-    if (ws->boxTimeout < 0)
     {
-	ws->boxTimeout = 0;
+	ws->boxTimeout = MAX (0, ws->boxTimeout);
 	damageScreen (s);
     }
 
+    if (!ws->moving && !ws->showPreview && ws->grabIndex)
+    {
+	removeScreenGrab (s, ws->grabIndex, NULL);
+	ws->grabIndex = 0;
+    }
+
     UNWRAP (ws, s, donePaintScreen);
     (*s->donePaintScreen) (s);
     WRAP (ws, s, donePaintScreen, wallDonePaintScreen);
@@ -1880,8 +1915,11 @@ wallInitScreen (CompPlugin *p,
 
     ws->boxTimeout = 0;
     ws->grabIndex = 0;
+    ws->timer = 0;
     ws->moving = FALSE;
     ws->showPreview = FALSE;
+    ws->focusDefault = TRUE;
+    ws->moveWindow = None;
 
     memset (&ws->switcherContext, 0, sizeof (WallCairoContext));
     memset (&ws->thumbContext, 0, sizeof (WallCairoContext));
@@ -1909,6 +1947,9 @@ wallFiniScreen (CompPlugin *p,
 {
     WALL_SCREEN (s);
 
+    if (ws->grabIndex)
+	removeScreenGrab (s, ws->grabIndex, NULL);
+
     wallDestroyCairoContext (s, &ws->switcherContext);
     wallDestroyCairoContext (s, &ws->thumbContext);
     wallDestroyCairoContext (s, &ws->highlightContext);
